/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2009 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: AccountingDatasourceGeneral.java 5103 2013-09-04 09:10:24Z tamas $ 
 * 
 */

package com.trackplus.plugin.report.accounting;

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import com.aurel.track.admin.customize.account.AccountBL;
import com.aurel.track.admin.customize.category.report.execute.ReportBeansToXML;
import com.aurel.track.admin.customize.lists.systemOption.IssueTypeBL;
import com.aurel.track.admin.customize.lists.systemOption.StatusBL;
import com.aurel.track.admin.project.ProjectAccountingTO;
import com.aurel.track.admin.project.ProjectBL;
import com.aurel.track.admin.user.person.PersonBL;
import com.aurel.track.beans.TAccountBean;
import com.aurel.track.beans.TCostBean;
import com.aurel.track.beans.TCostCenterBean;
import com.aurel.track.beans.TListTypeBean;
import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TProjectBean;
import com.aurel.track.beans.TStateBean;
import com.aurel.track.beans.TWorkItemBean;
import com.aurel.track.item.budgetCost.ExpenseBL;
import com.aurel.track.json.JSONUtility;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.PropertiesHelper;
import com.aurel.track.util.StringArrayParameterUtils;
import com.aurel.track.util.EmailHandling.Html2Text;
import com.aurel.track.util.numberFormatter.DoubleWithDecimalDigitsNumberFormatUtil;


public class AccountingDatasourceGeneral extends BudgetExpenseDatasourceBase {

	private static Logger logger = Logger.getLogger(AccountingDatasourceGeneral.class);
	
	public static interface ACCOUNTING_PARAMETER_NAME {
		public static String COSTCENTER = "costcenters";
		public static String COSTCENTER_OPTIONS = "costcenter" + PARAMETER_NAME.OPTION_SUFFIX;
		public static String COSTCENTERS_NAME_FIELD = COSTCENTER + PARAMETER_NAME.NAME_SUFFIX;
		public static String COSTCENTERS_NAME_VALUE = PARAMETER_NAME.MAP_PREFIX + COSTCENTER;
		
		public static String ACCOUNTS = "accounts";
		public static String ACCOUNT_OPTIONS = "account" + PARAMETER_NAME.OPTION_SUFFIX;
		public static String ACCOUNTS_NAME_FIELD = ACCOUNTS + PARAMETER_NAME.NAME_SUFFIX;
		public static String ACCOUNTS_NAME_VALUE = PARAMETER_NAME.MAP_PREFIX + ACCOUNTS;
	}
	
	private static Integer ALL_COSTCENTERS = Integer.valueOf(0);
	
	/** 
	 * The defined operation modes
	 * 0 - overview of own costs
	 * 1 - overview of all costs
	 */
	protected static class OperationMode {
		static public final int PERSONALOVERVIEW = 0;
		static public final int MANAGEROVERVIEW = 1;
	}
	
	/**
	 * Gets the data source object (a Document object in this case) retrieved using the parameters settings
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param templateDescriptionMap
	 * @param templateID
	 * @param personBean
	 * @param locale
	 * @return
	 */
	public Object getDatasource(Map<String, String[]> parameters, DatasourceDescriptor datasourceDescriptor,
			Map<String, Object> contextMap, Map<String, Object> templateDescriptionMap,
			Integer templateID, TPersonBean personBean, Locale locale) {
		Map<String, Object> savedParamsMap = new HashMap<String, Object>();
		String paramSettings = loadParamObjectsAndPropertyStringsAndFromStringArrParams(parameters, locale, savedParamsMap);
		int selectedOperationMode = StringArrayParameterUtils.parseIntegerValue(parameters, 
				BUGET_EXENSE_PARAMETER_NAME.OVERVIEW, OperationMode.PERSONALOVERVIEW);
		Integer[] selectedPersonsArr;
		if (selectedOperationMode == OperationMode.PERSONALOVERVIEW) {
			selectedPersonsArr = new Integer[] {personBean.getObjectID()};
		} else {
			selectedPersonsArr = StringArrayParameterUtils.parseIntegerArrValue(
				parameters, BUGET_EXENSE_PARAMETER_NAME.PERSON);
		}
		List<Integer> selectedAccounts = null;
		Integer[] selectedAccountIDs = StringArrayParameterUtils.parseIntegerArrValue(
				parameters, ACCOUNTING_PARAMETER_NAME.ACCOUNTS);
		Integer selectedCostcenterID = StringArrayParameterUtils.parseIntegerValue(parameters, 
				ACCOUNTING_PARAMETER_NAME.COSTCENTER);
		if (selectedAccountIDs!=null && selectedAccountIDs.length>0) {
			selectedAccounts = GeneralUtils.createListFromIntArr(selectedAccountIDs);
		} else {
			if (selectedCostcenterID!=null) {
				List<TAccountBean> accountsByCostCenters = AccountBL.loadByCostcenter(selectedCostcenterID);
				selectedAccounts = GeneralUtils.createIntegerListFromBeanList(accountsByCostCenters);
			}
		}
		//saves the parameters into the database
		paramSettings = PropertiesHelper.setIntegerProperty(paramSettings, BUGET_EXENSE_PARAMETER_NAME.OVERVIEW, selectedOperationMode);
		paramSettings = PropertiesHelper.setIntegerArrProperty(paramSettings, BUGET_EXENSE_PARAMETER_NAME.PERSON, selectedPersonsArr);
		paramSettings = PropertiesHelper.setIntegerProperty(paramSettings, ACCOUNTING_PARAMETER_NAME.COSTCENTER, selectedCostcenterID);
		paramSettings = PropertiesHelper.setProperty(paramSettings, ACCOUNTING_PARAMETER_NAME.ACCOUNTS,
				GeneralUtils.createCommaSeparatedStringFromIntegerArr(selectedAccountIDs));
		saveParameters(paramSettings, personBean.getObjectID(), templateID);
		//gets the datasource
		Integer datasourceType = (Integer)savedParamsMap.get(PARAMETER_NAME.DATASOURCETYPE);
		Integer projectOrReleaseID = (Integer)savedParamsMap.get(PARAMETER_NAME.PROJECT_OR_RELEASE_ID);
		Integer filterID = (Integer)savedParamsMap.get(PARAMETER_NAME.FILTER_ID);
		Date dateFrom = (Date)savedParamsMap.get(TIME_PERIOD_PARAMETER_NAME.DATE_FROM);
		Date dateTo = (Date)savedParamsMap.get(TIME_PERIOD_PARAMETER_NAME.DATE_TO);
		List<Map<String, String>> expensesList = getExpensesList(datasourceType, projectOrReleaseID, filterID,
				dateFrom, dateTo, contextMap, selectedPersonsArr, selectedAccounts, personBean, locale);
		return convertToDOM(expensesList, getFieldNameList(), personBean.getFullName(), locale);
	}

	public void serializeDatasource(OutputStream outputStream, Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}
	
	/**
	 * Prepares a map for rendering the config page
	 * @param savedParamsMap
	 * @param datasourceDescriptor
	 * @param personBean
	 * @param locale 
	 * @return
	 */
	protected String getBudgetExpenseExtraParams(Map<String, Object> savedParamsMap,
			DatasourceDescriptor datasourceDescriptor,
			TPersonBean personBean, Locale locale) {
		StringBuilder stringBuilder = new StringBuilder();
		List<TCostCenterBean> costCenters = AccountBL.getAllCostcenters();
		TCostCenterBean costCenterBean = new TCostCenterBean();
		costCenterBean.setCostcenterNumber("-");
		costCenters.add(0, costCenterBean);
		JSONUtility.appendILabelBeanList(stringBuilder, ACCOUNTING_PARAMETER_NAME.COSTCENTER_OPTIONS, (List)costCenters);
		Integer selectedCostcenter = (Integer)savedParamsMap.get(ACCOUNTING_PARAMETER_NAME.COSTCENTER);
		JSONUtility.appendIntegerValue(stringBuilder, ACCOUNTING_PARAMETER_NAME.COSTCENTER, selectedCostcenter);
		JSONUtility.appendStringValue(stringBuilder, ACCOUNTING_PARAMETER_NAME.COSTCENTERS_NAME_FIELD, ACCOUNTING_PARAMETER_NAME.COSTCENTERS_NAME_VALUE);
		List<TAccountBean> accountsList;
		if (selectedCostcenter==null) {
			accountsList = AccountBL.getAllAccounts();
		} else {
			accountsList = AccountBL.loadByCostcenter(selectedCostcenter);
		}
		JSONUtility.appendILabelBeanList(stringBuilder, ACCOUNTING_PARAMETER_NAME.ACCOUNT_OPTIONS, (List)accountsList);
		JSONUtility.appendIntegerArrayAsArray(stringBuilder, ACCOUNTING_PARAMETER_NAME.ACCOUNTS,
				(Integer[])savedParamsMap.get(ACCOUNTING_PARAMETER_NAME.ACCOUNTS));
		JSONUtility.appendStringValue(stringBuilder, ACCOUNTING_PARAMETER_NAME.ACCOUNTS_NAME_FIELD, ACCOUNTING_PARAMETER_NAME.ACCOUNTS_NAME_VALUE, true);		
		return stringBuilder.toString();
	}
	
	/**
	 * 
	 * @param paramSettings
	 * @return
	 */
	protected Map<String, Object> loadParamObjectsFromPropertyStrings(String paramSettings) {
		Map<String, Object> paramsMap = super.loadParamObjectsFromPropertyStrings(paramSettings);
		if (paramSettings!=null) {
			Integer costcenter = PropertiesHelper.getIntegerProperty(paramSettings, ACCOUNTING_PARAMETER_NAME.COSTCENTER);
			paramsMap.put(ACCOUNTING_PARAMETER_NAME.COSTCENTER, costcenter);
			String accountsArr = PropertiesHelper.getProperty(paramSettings, ACCOUNTING_PARAMETER_NAME.ACCOUNTS);
			Integer[] accounts = GeneralUtils.createIntegerArrFromCommaSeparatedString(accountsArr);
			paramsMap.put(ACCOUNTING_PARAMETER_NAME.ACCOUNTS, accounts);
		} else {
			paramsMap.put(BUGET_EXENSE_PARAMETER_NAME.OVERVIEW, OperationMode.PERSONALOVERVIEW);
		}
		return paramsMap;
	}
	
	/**
	 * Refreshing of some parameters through AJAX: only a part of the parameters should be recalculated 
	 * @param parameters
	 * @param templateID
	 * @param datasourceDescriptor
	 * @param contextMap
	 * @param personBean
	 * @param locale 
	 * @return
	 */
	@Override
	public String refreshParameters(Map<String, String[]> parameters, Integer templateID,
			DatasourceDescriptor datasourceDescriptor, Map<String, Object> contextMap,
			TPersonBean personBean, Locale locale) {
		List<TAccountBean> accountsByCostCenters;
		Integer selectedCostcenterID = StringArrayParameterUtils.parseIntegerValue(parameters, 
				ACCOUNTING_PARAMETER_NAME.COSTCENTER);
        if (selectedCostcenterID!=null) {
            if (!ALL_COSTCENTERS.equals(selectedCostcenterID)) {
                accountsByCostCenters = AccountBL.loadByCostcenter(selectedCostcenterID);
            } else {
                accountsByCostCenters = AccountBL.getAllAccounts();
            }
            return JSONUtility.encodeILabelBeanList((List)accountsByCostCenters);
        }
        //something else was refreshed
        return super.refreshParameters(parameters, templateID, datasourceDescriptor, contextMap, personBean, locale);
	}
	
	
	private Document convertToDOM(List<Map<String, String>> expensesList, List<String> fieldNames, String personName, Locale locale) {
		Document dom = null;
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			DocumentBuilder builder = factory.newDocumentBuilder ();
			dom = builder.newDocument ();	
		} catch (FactoryConfigurationError e){
			logger.error ("Creating the DOM document failed with FactoryConfigurationError:" + e.getMessage());
			return null;
		} catch (ParserConfigurationException e){
			logger.error ("Creating the DOM document failed with ParserConfigurationException: " + e.getMessage());
			return null;
		}
		Element root = dom.createElement ("expense-report");
		Element createdBy =createDomElement("createdBy",personName, false, dom);
		if (createdBy!=null) {
			root.appendChild(createdBy);
		}
		if (expensesList!=null) {
			Iterator<Map<String, String>> itrExpenses = expensesList.iterator();
			while (itrExpenses.hasNext()) {
				Map<String, String> expenseMap = itrExpenses.next();
				Element expenseElement = createDomElement("expense", null, false, dom);
				root.appendChild(expenseElement);
				Iterator<String> itrFieldNames = fieldNames.iterator();
				while (itrFieldNames.hasNext()) {
					String fieldName = itrFieldNames.next();
					expenseElement.appendChild(createDomElement(fieldName, expenseMap.get(fieldName), false, dom));
				}
			}
		}
		dom.appendChild(root);
		return dom;
	 }
	
	 /**
	 * Create a dom element with text content and attributes
	 * @param elementName
	 * @param textContent
	 * @param needCDATA
	 * @param dom
	 * @return
	 */
	private static Element createDomElement(String elementName, String textContent, boolean needCDATA, Document dom) {
		//create the main element
		Element element = null;
		try {
			element = dom.createElement(elementName);
		} catch (DOMException e) {
			logger.warn("Creating an XML node with the element name " + elementName + " failed with " + e);
		}
		if (element==null) {
			return null;
		}
		
		//create the text element
		if (textContent!=null) {
			if (needCDATA) {
				CDATASection cdataText = dom.createCDATASection(Html2Text.stripNonValidXMLCharacters(textContent));
				element.appendChild(cdataText);
			} else {
				Text textElement;
				try {
					textElement = dom.createTextNode(textContent);
				} catch (DOMException e) {
					logger.info("Creating the text node for the element " + elementName + " and text " + textContent + " failed with " + e);
					textElement = dom.createTextNode("");
				}
				//append the text to the element
				element.appendChild(textElement);
			}
		}
		return element;
	}
	
	private List<Map<String, String>> getExpensesList(Integer datasourceType,
			Integer projectOrReleaseID, Integer filterID, Date dateFrom, Date dateTo,
			 Map<String, Object> contextMap, Integer[] selectedPersonIDsArr, 
			List<Integer> selectedAccounts, TPersonBean personBean, Locale locale) {
		List<Map<String, String>> expensesList = new ArrayList<Map<String,String>>();
		Set<Integer> projectSet = new HashSet<Integer>();
		List<Integer> workItemIDList = new LinkedList<Integer>(); 
		List<TWorkItemBean> workItemBeanList = getWorkItemBeans(datasourceType, projectOrReleaseID, 
				filterID, contextMap, true, true, true, personBean, locale);
		Map<Integer, TWorkItemBean> workItemBeansMap = new HashMap<Integer, TWorkItemBean>();
		for (TWorkItemBean workItemBean : workItemBeanList) {
			Integer workItemID = workItemBean.getObjectID();
			workItemIDList.add(workItemID);
			workItemBeansMap.put(workItemID, workItemBean);
			projectSet.add(workItemBean.getProjectID());
		}
		List<TCostBean> costBeansList = ExpenseBL.loadCostsForWorkItems(GeneralUtils.createIntArrFromIntegerList(workItemIDList),
				workItemBeansMap, dateFrom, dateTo, personBean.getObjectID(), selectedPersonIDsArr, selectedAccounts);
		if (costBeansList!=null && costBeansList.size()>0) {
			Set<Integer> personIDSet = new HashSet<Integer>();
			for (TCostBean costBean : costBeansList) {
				personIDSet.add(costBean.getPerson());
			}
			Map<Integer, TPersonBean> personBeanMap = GeneralUtils.createMapFromList(PersonBL.loadByKeys(GeneralUtils.createIntegerListFromCollection(personIDSet)));
			Map<Integer, TAccountBean> accountBeanMap;
			if (selectedAccounts==null || selectedAccounts.size()==0) {
				accountBeanMap = GeneralUtils.createMapFromList(AccountBL.getAllAccounts());
			} else {
				accountBeanMap = GeneralUtils.createMapFromList(AccountBL.loadAccountsByKeys(selectedAccounts));
			}
			Map<Integer, TCostCenterBean> costcenterBeanMap = GeneralUtils.createMapFromList(AccountBL.getAllCostcenters());
			List<TProjectBean> projectBeansList = ProjectBL.loadByProjectIDs(GeneralUtils.createListFromSet(projectSet));
			Map<Integer, TProjectBean> projectBeansMap = GeneralUtils.createMapFromList(projectBeansList);
			Map<Integer, ProjectAccountingTO> accountingAttributesMap = ProjectBL.getAccountingAttributesMap(projectSet);
			Map<Integer, TStateBean> statusBeanMap = GeneralUtils.createMapFromList(StatusBL.loadAll());
			Map<Integer, TListTypeBean> issueTypeBeanMap = GeneralUtils.createMapFromList(IssueTypeBL.loadAll());
			Iterator<TCostBean> itrCostBean = costBeansList.iterator();
			while (itrCostBean.hasNext()) {
				TCostBean costBean = itrCostBean.next();
				Map<String, String> expenseMap = new HashMap<String, String>();
				//locale is english because we need the iso format
				if (costBean.getHours()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.HOURS,
						DoubleWithDecimalDigitsNumberFormatUtil.getInstance(2).formatGUI(costBean.getHours(), Locale.ENGLISH));
				}
				if (costBean.getCost()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.COST, 
						DoubleWithDecimalDigitsNumberFormatUtil.getInstance(2).formatGUI(costBean.getCost(), Locale.ENGLISH));
				}
				expenseMap.put(ACCOUNTING_FIELDNAMES.SUBJECT, costBean.getSubject());
				expenseMap.put(ACCOUNTING_FIELDNAMES.EFFORTDATE, DateTimeUtils.getInstance().formatISODateTime(costBean.getEffortdate()));
				expenseMap.put(ACCOUNTING_FIELDNAMES.DESCRIPTION, costBean.getDescription());
				expenseMap.put(ACCOUNTING_FIELDNAMES.LASTEDIT, DateTimeUtils.getInstance().formatISODateTime(costBean.getLastEdit()));
				if (costBean.getAccount()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNT, costBean.getAccount().toString());
					TAccountBean accountBean = accountBeanMap.get(costBean.getAccount());
					if (accountBean!=null) {
						expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNTNAME, accountBean.getAccountName());
						expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNTNUMBER, accountBean.getAccountNumber());
						if (accountBean.getStatus()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNTSTATUS, accountBean.getStatus().toString());
						}
						if (accountBean.getCostCenter()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.COSTCENTER, accountBean.getCostCenter().toString());
							TCostCenterBean costCenterBean = costcenterBeanMap.get(accountBean.getCostCenter());
							if (costCenterBean!=null) {
								expenseMap.put(ACCOUNTING_FIELDNAMES.COSTCENTERNAME, costCenterBean.getCostcenterName());
								expenseMap.put(ACCOUNTING_FIELDNAMES.COSTCENTERNUMBER, costCenterBean.getCostcenterNumber());
							}
						}
					}
				}
				if (costBean.getPerson()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.PERSON, costBean.getPerson().toString());
					TPersonBean costPersonBean = personBeanMap.get(costBean.getPerson());
					if (costPersonBean!=null) {
						expenseMap.put(ACCOUNTING_FIELDNAMES.PERSONNAME, costPersonBean.getFullName());
					}
				}
				if (costBean.getWorkitem()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.WORKITEM, costBean.getWorkitem().toString());
					TWorkItemBean workItemBean = workItemBeansMap.get(costBean.getWorkitem());
					if (workItemBean!=null) {
						expenseMap.put(ACCOUNTING_FIELDNAMES.WORKITEMTITLE, workItemBean.getSynopsis());
							TProjectBean projectBean = projectBeansMap.get(workItemBean.getProjectID());
							ProjectAccountingTO projectAccountingTO = accountingAttributesMap.get(projectBean.getObjectID());
							if (projectBean!=null) {
								expenseMap.put(ACCOUNTING_FIELDNAMES.PROJECT, projectBean.getObjectID().toString());
								expenseMap.put(ACCOUNTING_FIELDNAMES.PROJECTLABEL, projectBean.getLabel());
								expenseMap.put(ACCOUNTING_FIELDNAMES.CURRENCYSYMBOL, projectAccountingTO.getCurrencySymbol());
								expenseMap.put(ACCOUNTING_FIELDNAMES.CURRENCYNAME, projectAccountingTO.getCurrencyName());
							}
						if (workItemBean.getStateID()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.STATUS, workItemBean.getStateID().toString());
							TStateBean stateBean = statusBeanMap.get(workItemBean.getStateID());
							if (stateBean!=null) {
								expenseMap.put(ACCOUNTING_FIELDNAMES.STATUSLABEL, stateBean.getLabel());
							}
						}
						if (workItemBean.getListTypeID()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.ISSUETYPE, workItemBean.getListTypeID().toString());
							TListTypeBean issueTypeBean = issueTypeBeanMap.get(workItemBean.getListTypeID());
							if (issueTypeBean!=null) {
								expenseMap.put(ACCOUNTING_FIELDNAMES.ISSUETYPELABEL, issueTypeBean.getLabel());
							}
						}
					}
				}
				expensesList.add(expenseMap);
			}
		}
		return expensesList;
	}
	
	private static List<String> getFieldNameList() {
		List<String> fieldNames = new ArrayList<String>();
		fieldNames.add(ACCOUNTING_FIELDNAMES.ACCOUNT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.PERSON);
		fieldNames.add(ACCOUNTING_FIELDNAMES.WORKITEM);
		fieldNames.add(ACCOUNTING_FIELDNAMES.HOURS);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COST);
		fieldNames.add(ACCOUNTING_FIELDNAMES.SUBJECT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.EFFORTDATE);
		fieldNames.add(ACCOUNTING_FIELDNAMES.DESCRIPTION);
		fieldNames.add(ACCOUNTING_FIELDNAMES.LASTEDIT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.ACCOUNTNAME);
		fieldNames.add(ACCOUNTING_FIELDNAMES.ACCOUNTNUMBER);
		fieldNames.add(ACCOUNTING_FIELDNAMES.STATUS);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COSTCENTER);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COSTCENTERNAME);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COSTCENTERNUMBER);
		fieldNames.add(ACCOUNTING_FIELDNAMES.PERSONNAME);
		fieldNames.add(ACCOUNTING_FIELDNAMES.WORKITEMTITLE);
		fieldNames.add(ACCOUNTING_FIELDNAMES.STATUS);
		fieldNames.add(ACCOUNTING_FIELDNAMES.ISSUETYPE);
		fieldNames.add(ACCOUNTING_FIELDNAMES.PROJECT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.PROJECTLABEL);
		fieldNames.add(ACCOUNTING_FIELDNAMES.CURRENCYSYMBOL);
		fieldNames.add(ACCOUNTING_FIELDNAMES.CURRENCYNAME);
		fieldNames.add(ACCOUNTING_FIELDNAMES.STATUSLABEL);
		fieldNames.add(ACCOUNTING_FIELDNAMES.ISSUETYPELABEL);
		return fieldNames;
	}
	
	private interface ACCOUNTING_FIELDNAMES {
		String ACCOUNT = "accountID";
		String PERSON = "personID";
		String WORKITEM = "workItemID";
		String HOURS = "hours";
		String COST = "cost";
		String SUBJECT = "subject";
		String EFFORTDATE = "effortDate";
		String DESCRIPTION = "description";
		String LASTEDIT = "lastEdit";
		
		String ACCOUNTNAME = "accountName";
		String ACCOUNTNUMBER = "accountNumber";
		String ACCOUNTSTATUS = "accountStatus";
		
		String COSTCENTER = "costcenterID";
		String COSTCENTERNAME = "costcenterName";
		String COSTCENTERNUMBER = "costcenterNumber";
		
		String PERSONNAME = "personName";
		
		String WORKITEMTITLE = "workItemTitle";
		String STATUS = "status";
		String ISSUETYPE = "issueType";
		
		String PROJECT = "projectID";
		String PROJECTLABEL= "projectLabel";
		String CURRENCYSYMBOL = "currencySymbol";
		String CURRENCYNAME = "currencyName";
		
		String STATUSLABEL= "statusLabel";
		
		String ISSUETYPELABEL = "issueTypeLabel";
	}
}
